@@ -1,23 +1,28 @@
 """
 codegen.py generates pika/spec.py
 
+The required spec json file can be found at
+https://github.com/rabbitmq/rabbitmq-codegen
+.
+
+After cloning it run the following to generate a spec.py file:
+python2 ./codegen.py ../../rabbitmq-codegen
 """
 from __future__ import nested_scopes
 
 import os
+import re
 import sys
 
-RABBITMQ_PUBLIC_UMBRELLA = '../../rabbitmq-public-umbrella'
-RABBITMQ_CODEGEN = 'rabbitmq-codegen'
-PIKA_SPEC = '../pika/spec.py'
+if sys.version_info.major != 2:
+    sys.exit('Python 2 is required at this time')
 
-CODEGEN_PATH = os.path.realpath('%s/%s' % (RABBITMQ_PUBLIC_UMBRELLA,
-                                           RABBITMQ_CODEGEN))
-print('codegen-path: %s' % CODEGEN_PATH)
-sys.path.append(CODEGEN_PATH)
+RABBITMQ_CODEGEN_PATH = sys.argv[1]
+PIKA_SPEC = '../pika/spec.py'
+print('codegen-path: %s' % RABBITMQ_CODEGEN_PATH)
+sys.path.append(RABBITMQ_CODEGEN_PATH)
 
 import amqp_codegen
-import re
 
 DRIVER_METHODS = {
     "Exchange.Bind": ["Exchange.BindOk"],
@@ -44,6 +49,10 @@ DRIVER_METHODS = {
 def fieldvalue(v):
     if isinstance(v, unicode):
         return repr(v.encode('ascii'))
+    elif isinstance(v, dict):
+        return repr(None)
+    elif isinstance(v, list):
+        return repr(None)
     else:
         return repr(v)
 
@@ -58,6 +67,8 @@ def pyize(s):
     s = normalize_separators(s)
     if s in ('global', 'class'):
         s += '_'
+    if s == 'global_':
+        s = 'global_qos'
     return s
 
 
@@ -65,8 +76,7 @@ def camel(s):
     return normalize_separators(s).title().replace('_', '')
 
 
-amqp_codegen.AmqpMethod.structName = lambda m: camel(
-    m.klass.name) + '.' + camel(m.name)
+amqp_codegen.AmqpMethod.structName = lambda m: camel(m.klass.name) + '.' + camel(m.name)
 amqp_codegen.AmqpClass.structName = lambda c: camel(c.name) + "Properties"
 
 
@@ -87,7 +97,9 @@ def generate(specPath):
     def genSingleDecode(prefix, cLvalue, unresolved_domain):
         type = spec.resolveDomain(unresolved_domain)
         if type == 'shortstr':
-            print(prefix + "%s, offset = data.decode_short_string(encoded, offset)" % cLvalue)
+            print(prefix +
+                  "%s, offset = data.decode_short_string(encoded, offset)" %
+                  cLvalue)
         elif type == 'longstr':
             print(prefix +
                   "length = struct.unpack_from('>I', encoded, offset)[0]")
@@ -99,43 +111,47 @@ def generate(specPath):
             print(prefix + "    pass")
             print(prefix + "offset += length")
         elif type == 'octet':
-            print(prefix + "%s = struct.unpack_from('B', encoded, offset)[0]" %
-                  cLvalue)
+            print(prefix +
+                  "%s = struct.unpack_from('B', encoded, offset)[0]" % cLvalue)
             print(prefix + "offset += 1")
         elif type == 'short':
-            print(prefix + "%s = struct.unpack_from('>H', encoded, offset)[0]" %
-                  cLvalue)
+            print(prefix +
+                  "%s = struct.unpack_from('>H', encoded, offset)[0]" % cLvalue)
             print(prefix + "offset += 2")
         elif type == 'long':
-            print(prefix + "%s = struct.unpack_from('>I', encoded, offset)[0]" %
-                  cLvalue)
+            print(prefix +
+                  "%s = struct.unpack_from('>I', encoded, offset)[0]" % cLvalue)
             print(prefix + "offset += 4")
         elif type == 'longlong':
-            print(prefix + "%s = struct.unpack_from('>Q', encoded, offset)[0]" %
-                  cLvalue)
+            print(prefix +
+                  "%s = struct.unpack_from('>Q', encoded, offset)[0]" % cLvalue)
             print(prefix + "offset += 8")
         elif type == 'timestamp':
-            print(prefix + "%s = struct.unpack_from('>Q', encoded, offset)[0]" %
-                  cLvalue)
+            print(prefix +
+                  "%s = struct.unpack_from('>Q', encoded, offset)[0]" % cLvalue)
             print(prefix + "offset += 8")
         elif type == 'bit':
             raise Exception("Can't decode bit in genSingleDecode")
         elif type == 'table':
-            print(Exception(prefix + "(%s, offset) = data.decode_table(encoded, offset)" % \
-                  cLvalue))
+            print(
+                Exception(prefix +
+                          "(%s, offset) = data.decode_table(encoded, offset)" %
+                          cLvalue))
         else:
             raise Exception("Illegal domain in genSingleDecode", type)
 
     def genSingleEncode(prefix, cValue, unresolved_domain):
         type = spec.resolveDomain(unresolved_domain)
         if type == 'shortstr':
-            print(prefix + \
-                "assert isinstance(%s, str_or_bytes),\\\n%s       'A non-string value was supplied for %s'" \
+            print(
+                prefix +
+                "assert isinstance(%s, str_or_bytes),\\\n%s       'A non-string value was supplied for %s'"
                 % (cValue, prefix, cValue))
             print(prefix + "data.encode_short_string(pieces, %s)" % cValue)
         elif type == 'longstr':
-            print(prefix + \
-                "assert isinstance(%s, str_or_bytes),\\\n%s       'A non-string value was supplied for %s'" \
+            print(
+                prefix +
+                "assert isinstance(%s, str_or_bytes),\\\n%s       'A non-string value was supplied for %s'"
                 % (cValue, prefix, cValue))
             print(
                 prefix +
@@ -171,9 +187,10 @@ def generate(specPath):
                     bitindex = 0
                 if not bitindex:
                     print(
-                        "            bit_buffer = struct.unpack_from('B', encoded, offset)[0]")
+                        "            bit_buffer = struct.unpack_from('B', encoded, offset)[0]"
+                    )
                     print("            offset += 1")
-                print("            self.%s = (bit_buffer & (1 << %d)) != 0" % \
+                print("            self.%s = (bit_buffer & (1 << %d)) != 0" %
                       (pyize(f.name), bitindex))
                 bitindex += 1
             else:
@@ -189,17 +206,19 @@ def generate(specPath):
         print("        flagword_index = 0")
         print("        while True:")
         print(
-            "            partial_flags = struct.unpack_from('>H', encoded, offset)[0]")
+            "            partial_flags = struct.unpack_from('>H', encoded, offset)[0]"
+        )
         print("            offset += 2")
         print(
-            "            flags = flags | (partial_flags << (flagword_index * 16))")
+            "            flags = flags | (partial_flags << (flagword_index * 16))"
+        )
         print("            if not (partial_flags & 1):")
         print("                break")
         print("            flagword_index += 1")
         for f in c.fields:
             if spec.resolveDomain(f.domain) == 'bit':
-                print("        self.%s = (flags & %s) != 0" %
-                      (pyize(f.name), flagName(c, f)))
+                print("        self.%s = (flags & %s) != 0" % (pyize(f.name),
+                                                               flagName(c, f)))
             else:
                 print("        if flags & %s:" % (flagName(c, f),))
                 genSingleDecode("            ", "self.%s" % (pyize(f.name),),
@@ -228,8 +247,7 @@ def generate(specPath):
                     print("            bit_buffer = 0")
                     bitindex = 0
                 print("            if self.%s:" % pyize(f.name))
-                print("                bit_buffer = bit_buffer | (1 << %d)" % \
-                    bitindex)
+                print("                bit_buffer |= 1 << %d" % bitindex)
                 bitindex += 1
             else:
                 finishBits()
@@ -246,8 +264,8 @@ def generate(specPath):
         print("        flags = 0")
         for f in c.fields:
             if spec.resolveDomain(f.domain) == 'bit':
-                print("        if self.%s: flags = flags | %s" %
-                      (pyize(f.name), flagName(c, f)))
+                print("        if self.%s: flags = flags | %s" % (pyize(
+                    f.name), flagName(c, f)))
             else:
                 print("        if self.%s is not None:" % (pyize(f.name),))
                 print("            flags = flags | %s" % (flagName(c, f),))
@@ -268,8 +286,10 @@ def generate(specPath):
         print('')
 
     def fieldDeclList(fields):
-        return ''.join([", %s=%s" % (pyize(f.name), fieldvalue(f.defaultvalue))
-                        for f in fields])
+        return ''.join([
+            ", %s=%s" % (pyize(f.name), fieldvalue(f.defaultvalue))
+            for f in fields
+        ])
 
     def fieldInitList(prefix, fields):
         if fields:
@@ -332,15 +352,12 @@ str = bytes
             print('    class %s(amqp_object.Method):' % (camel(m.name),))
             print('')
             methodid = m.klass.index << 16 | m.index
-            print("        INDEX = 0x%.08X  # %d, %d; %d" % \
-                  (methodid,
-                                                                                 m.klass.index,
-                                                                                 m.index,
-                                                                                 methodid))
+            print("        INDEX = 0x%.08X  # %d, %d; %d" %
+                  (methodid, m.klass.index, m.index, methodid))
             print("        NAME = %s" % (fieldvalue(m.structName(),)))
             print('')
-            print("        def __init__(self%s):" %
-                  (fieldDeclList(m.arguments),))
+            print(
+                "        def __init__(self%s):" % (fieldDeclList(m.arguments),))
             print(fieldInitList('            ', m.arguments))
             print("        @property")
             print("        def synchronous(self):")
@@ -377,15 +394,19 @@ str = bytes
             genEncodeProperties(c)
 
     print("methods = {")
-    print(',\n'.join(["    0x%08X: %s" % (m.klass.index << 16 | m.index, m.structName()) \
-                      for m in spec.allMethods()]))
+    print(',\n'.join([
+        "    0x%08X: %s" % (m.klass.index << 16 | m.index, m.structName())
+        for m in spec.allMethods()
+    ]))
     print("}")
     print('')
 
     print("props = {")
-    print(',\n'.join(["    0x%04X: %s" % (c.index, c.structName()) \
-                      for c in spec.allClasses() \
-                      if c.fields]))
+    print(',\n'.join([
+        "    0x%04X: %s" % (c.index, c.structName())
+        for c in spec.allClasses()
+        if c.fields
+    ]))
     print("}")
     print('')
     print('')
@@ -401,4 +422,4 @@ str = bytes
 if __name__ == "__main__":
     with open(PIKA_SPEC, 'w') as handle:
         sys.stdout = handle
-        generate(['%s/amqp-rabbitmq-0.9.1.json' % CODEGEN_PATH])
+        generate(['%s/amqp-rabbitmq-0.9.1.json' % RABBITMQ_CODEGEN_PATH])
